''' <summary>
''' Module contenant les membres destins  toute l'application
''' </summary>
''' <remarks></remarks>
Friend Module MdlFriend

#Region "Enumrations"
    ''' <summary>
    ''' Enumration des popups affichables
    ''' </summary>
    ''' <remarks></remarks>
    Public Enum EPopups
        ''' <summary>
        ''' Aucune popup visible
        ''' </summary>
        ''' <remarks></remarks>
        Aucun = 0
        ''' <summary>
        ''' Popup en cas d'ajout d'un client
        ''' </summary>
        ''' <remarks></remarks>
        AddClient = 1
        ''' <summary>
        ''' Popup en cas de retrait d'un client
        ''' </summary>
        ''' <remarks></remarks>
        RmvClient = 2
        ''' <summary>
        ''' Popup  l'arrive d'un message
        ''' </summary>
        ''' <remarks></remarks>
        Message = 4
        ''' <summary>
        ''' Popup pour indiquer l'acquitement d'un message
        ''' </summary>
        ''' <remarks></remarks>
        Ack = 8
    End Enum
    ''' <summary>
    ''' Enumration des sons jouables
    ''' </summary>
    ''' <remarks></remarks>
    Public Enum ESound
        ''' <summary>
        ''' Aucune son
        ''' </summary>
        ''' <remarks></remarks>
        Aucun = 0
        ''' <summary>
        ''' Son en cas d'ajout d'un client
        ''' </summary>
        ''' <remarks></remarks>
        AddClient = 1
        ''' <summary>
        ''' Son en cas de retrait d'un client
        ''' </summary>
        ''' <remarks></remarks>
        RmvClient = 2
        ''' <summary>
        ''' Son  l'arrive d'un message
        ''' </summary>
        ''' <remarks></remarks>
        Message = 4
        ''' <summary>
        ''' Son pour indiquer l'acquitement d'un message
        ''' </summary>
        ''' <remarks></remarks>
        Ack = 8
        ''' <summary>
        ''' Son en cas de reconnexion au rseau
        ''' </summary>
        ''' <remarks></remarks>
        Connect = 16
        ''' <summary>
        ''' Son en cas de dconnexion du rseau
        ''' </summary>
        ''' <remarks></remarks>
        Deconnect = 32
    End Enum
#End Region

#Region "Accessibilit du rseau"
    ''' <summary>
    ''' Indique si le rseau est accessible ou non
    ''' </summary>
    ''' <remarks></remarks>
    Private mNetworkAvailable As Boolean = True
    ''' <summary>
    ''' Delegate pour l'event correspondant
    ''' </summary>
    ''' <remarks></remarks>
    Public Delegate Sub EvtNetworkAvailabilityChanged()
    ''' <summary>
    ''' Evenement gnr lors de changement de connectivit du rseau
    ''' </summary>
    ''' <remarks></remarks>
    Public Event NetworkAvailabilityChanged As EvtNetworkAvailabilityChanged
    ''' <summary>
    ''' Retourne ou dfinit la connectivit du rseau
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Property NetworkAvailable() As Boolean
        Get
            Return mNetworkAvailable
        End Get
        Set(ByVal value As Boolean)
            If mNetworkAvailable = value Then Exit Property
            mNetworkAvailable = value
            Try
                RaiseEvent NetworkAvailabilityChanged()
            Catch ex As Exception
                AddLog("MdlFriend.NetWorkAvailable", "Exception dans RaiseEvent : {0}", ex, EPrio.Avertissement)
#If DEBUG Then
                Stop
#End If
            End Try
        End Set
    End Property
#End Region

#Region "Opration sur les tableaux"
    ''' <summary>
    ''' Retourne un tableau d'octet correspondant  la donne passe en paramtre
    ''' </summary>
    ''' <param name="pData">Valeur  transcoder</param>
    ''' <returns></returns>

    Public Function GetByteArray(ByVal pData As Integer) As Byte()
        If pData < 0 Then
            Dim lUInt As UInteger = UInteger.MaxValue - CUInt(Math.Abs(pData + 1))
            Return GetByteArray(lUInt)
        Else
            Return GetByteArray(CUInt(pData))
        End If
    End Function
    ''' <summary>
    ''' Retourne un tableau d'octet correspondant  la donne passe en paramtre
    ''' </summary>
    ''' <param name="pData">Valeur  transcoder</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function GetByteArray(ByVal pData As UInteger) As Byte()
        Dim lTab(3) As Byte

        lTab(3) = CByte(pData And &HFF)
        lTab(2) = CByte((pData And &HFF00) >> 8)
        lTab(1) = CByte((pData And &HFF0000) >> 16)
        lTab(0) = CByte((pData And &HFF00000) >> 24)

        Return lTab
    End Function
    ''' <summary>
    ''' Retourne un tableau d'octet correspondant  la chaine passe en paramtre (avec null terminal)
    ''' </summary>
    ''' <param name="pData">Valeur  transcoder</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function GetByteArray(ByVal pData As String) As Byte()
        Dim lTab As New List(Of Byte)

        lTab.AddRange(System.Text.UTF8Encoding.UTF8.GetBytes(pData))
        If lTab.Count = 0 OrElse lTab(lTab.Count - 1) <> 0 Then lTab.Add(0)

        Return lTab.ToArray
    End Function
    ''' <summary>
    ''' Retourne un tableau d'octet correspondant  la donne passe en paramtre
    ''' </summary>
    ''' <param name="pData">Valeur  transcoder</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function GetByteArray(ByVal pData As Short) As Byte()
        If pData < 0 Then
            Dim lUInt As UShort = UShort.MaxValue - CUShort(Math.Abs(pData + 1))
            Return GetByteArray(lUInt)
        Else
            Return GetByteArray(CUShort(pData))
        End If
    End Function
    ''' <summary>
    ''' Retourne un tableau d'octet correspondant  la donne passe en paramtre
    ''' </summary>
    ''' <param name="pData">Valeur  transcoder</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function GetByteArray(ByVal pData As UShort) As Byte()
        Dim lTab(1) As Byte

        lTab(1) = CByte(pData And &HFF)
        lTab(0) = CByte((pData And &HFF00) >> 8)

        Return lTab
    End Function
    ''' <summary>
    ''' Retourne un entier sign  partir d'un tableau
    ''' </summary>
    ''' <param name="pData">Tableau de 4 octets</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function GetInt32(ByVal pData As List(Of Byte)) As Integer
        Dim lUInt As UInteger = GetUInt32(pData)
        Dim lInt As Integer = 0
        If lUInt > Integer.MaxValue Then
            lInt = -1 - CInt(UInteger.MaxValue - lUInt)
        Else
            lInt = CInt(lUInt)
        End If
        Return lInt
    End Function
    ''' <summary>
    ''' Retourne un entier non sign  partir d'un tableau
    ''' </summary>
    ''' <param name="pData">Tableau de 4 octets</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function GetUInt32(ByVal pData As List(Of Byte)) As UInteger
        Dim lInt As UInteger = pData(0)
        lInt = lInt << 8
        lInt += pData(1)
        lInt = lInt << 8
        lInt += pData(2)
        lInt = lInt << 8
        lInt += pData(3)
        pData.RemoveRange(0, 4)
        Return lInt
    End Function
    ''' <summary>
    ''' Retourne une string  partir d'un <c>List(of Byte)</c> et supprime les donnes utilises
    ''' </summary>
    ''' <param name="pData">Tableau source</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function GetString(ByVal pData As List(Of Byte)) As String
        Dim lStr As String = Nothing

        For i As Integer = 0 To pData.Count - 1
            If pData(i) = 0 Then
                lStr = System.Text.Encoding.UTF8.GetString(pData.GetRange(0, i).ToArray)
                pData.RemoveRange(0, i + 1)
                Exit For
            End If
        Next

        If lStr Is Nothing Then
            lStr = System.Text.Encoding.UTF8.GetString(pData.ToArray)
            pData.Clear()
        End If

        Return lStr
    End Function
    ''' <summary>
    ''' Retourne un tableau concatn
    ''' </summary>
    ''' <param name="pData">Donnes source</param>
    ''' <param name="pNbByte">Nombre d'octets  prendre du tableau source</param>
    ''' <param name="pAddData">Donnes  ajouter en fin de tableau</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function GetByteArray(ByVal pData As List(Of Byte), ByVal pNbByte As Integer, ByVal pAddData As Byte()) As Byte()
        Dim lTab As New List(Of Byte)

        lTab.AddRange(GetByteArray(pData, pNbByte))
        lTab.AddRange(pAddData)

        Return lTab.ToArray
    End Function
    ''' <summary>
    ''' Retourne une partie du tableau d'entre
    ''' </summary>
    ''' <param name="pData">Donnes sources</param>
    ''' <param name="pNbByte">Nombre d'octet  prendre</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function GetByteArray(ByVal pData As List(Of Byte), ByVal pNbByte As Integer) As Byte()
        Dim ltData As Byte()
        If pData.Count >= pNbByte Then
            ltData = pData.GetRange(0, pNbByte).ToArray
            pData.RemoveRange(0, 4)
        Else
            ltData = pData.GetRange(0, pData.Count).ToArray
            pData.Clear()
        End If
        Return ltData
    End Function
    ''' <summary>
    ''' Retourne un entier court sign  partir d'un tableau
    ''' </summary>
    ''' <param name="pData">Tableau de 2 octets</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function GetShort(ByVal pData As List(Of Byte)) As Short
        Dim lUInt As UShort = GetUshort(pData)
        Dim lInt As Short = 0
        If lUInt > Short.MaxValue Then
            lInt = CShort(-1) - CShort(UShort.MaxValue - lUInt)
        Else
            lInt = CShort(lUInt)
        End If
        Return lInt
    End Function
    ''' <summary>
    ''' Retourne un entier court non sign  partir d'un tableau
    ''' </summary>
    ''' <param name="pData">Tableau de 2 octets</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function GetUshort(ByVal pData As List(Of Byte)) As UShort
        Dim lInt As UShort = pData(0)
        lInt = lInt << 8
        lInt += pData(1)
        pData.RemoveRange(0, 2)
        Return lInt
    End Function
    ''' <summary>
    ''' Retourne un octet du tableau
    ''' </summary>
    ''' <param name="pTab">Tableau source</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function GetByte(ByVal pTab As List(Of Byte)) As Byte
        Dim lByte As Byte = pTab(0)
        pTab.RemoveAt(0)
        Return lByte
    End Function
#End Region

#Region "surcharge gnriques des mthodes de bases"
    ''' <summary>
    ''' Retourne une valeur
    ''' </summary>
    ''' <typeparam name="T">Type des donnes  retourner</typeparam>
    ''' <param name="pCond">Condition</param>
    ''' <param name="pValT">Valeur si vrai</param>
    ''' <param name="pValF">Valeur si faux</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function IIF(Of T)(ByVal pCond As Boolean, ByVal pValT As T, ByVal pValF As T) As T
        If pCond Then
            Return pValT
        Else
            Return pValF
        End If
    End Function
    ''' <summary>
    ''' Retourne la valeur minimale de 2 valeur
    ''' </summary>
    ''' <typeparam name="T">Type  comparer</typeparam>
    ''' <param name="pValue1">Valeur1</param>
    ''' <param name="pValue2">Valeur2</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function Min(Of T As IComparable)(ByVal pValue1 As T, ByVal pValue2 As T) As T
        If pValue1.CompareTo(pValue2) > 0 Then
            Return pValue2
        Else
            Return pValue1
        End If
    End Function
    ''' <summary>
    ''' Retourne la valeur maximale de 2 valeur
    ''' </summary>
    ''' <typeparam name="T">Type  comparer</typeparam>
    ''' <param name="pValue1">Valeur1</param>
    ''' <param name="pValue2">Valeur2</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function Max(Of T As IComparable)(ByVal pValue1 As T, ByVal pValue2 As T) As T
        If pValue1.CompareTo(pValue2) < 0 Then
            Return pValue2
        Else
            Return pValue1
        End If
    End Function
#End Region

#Region "Gestion des modifs des paramtres"
    'Gestion des changement des paramtres de l'application
    Friend Sub Setting_SettingChanging(ByVal sender As Object, ByVal e As System.Configuration.SettingChangingEventArgs)
        If My.Settings.Item(e.SettingName).Equals(e.NewValue) Then Exit Sub
        AddLog("Settings", "Paramtre ""{0}"" : Anc ""{1}"", New ""{2}""", e.SettingName, My.Settings.Item(e.SettingName), e.NewValue, EPrio.Information)
    End Sub
    'Chargement des paramtres
    Friend Sub Setting_SettingsLoaded(ByVal sender As Object, ByVal e As System.Configuration.SettingsLoadedEventArgs)
        Dim ltStr As New SortedList(Of String, String)

        For Each lProp As System.Configuration.SettingsProperty In My.Settings.Properties
            ltStr.Add(lProp.Name, String.Format("{0} ({1}) : {2}", lProp.Name, My.Settings.Item(lProp.Name).GetType.ToString, My.Settings.Item(lProp.Name)))
        Next

        Dim lStr As New List(Of String)
        lStr.AddRange(ltStr.Values)

        AddLog("Settings", "Settings : " + vbCrLf + " " + String.Join(vbCrLf + " ", lStr.ToArray), EPrio.Information)
    End Sub
#End Region
End Module
